﻿using System;
using System.Collections.Generic;
using KaoriYa.Migemo;
using System.Text.RegularExpressions;
using Tomham;

namespace CommandTagLauncher
{
	public class Helper
	{
		public Helper()
		{
			InitializeMigemo();
		}

		private Migemo migemo;

		private void InitializeMigemo()
		{
			try
			{
				migemo = new Migemo( @"./dict/migemo-dict" );
			}
			catch ( Exception e )
			{
				Console.WriteLine( e.Message );

				migemo = null;
			}
		}

		#region IsMatchedItem

		internal bool IsMatchedItem( Command command )
		{
			return IsMatchedItem( command, arguments );
		}

		private IsMatchedItemArguments arguments;

		private int maxPatternLength = 500;

		internal void PrepareIsMatchedItemArguments( string filter, bool initialSearch, bool romajiMatching )
		{
			string[] splittedFilters = SplitFilter( filter );

			IsMatchedItemArguments arguments = new IsMatchedItemArguments();

			arguments.InitialSearch = initialSearch;
			arguments.RomajiMatching = romajiMatching;

			foreach ( string splittedFilter in splittedFilters )
			{
				if ( IsTagFilter( splittedFilter ) )
				{
					string tag = splittedFilter.Substring( 1 );

					if ( tag != string.Empty )
					{
						arguments.Tags.Add( tag );
					}
				}
				else
				{
					arguments.Words.Add( splittedFilter );
				}
			}

			if ( romajiMatching && migemo != null )
			{
				foreach ( string word in arguments.Words )
				{
					string pattern = migemo.Query( word );

					if ( initialSearch )
					{
						pattern = @"^" + pattern;
					}

					if ( pattern.Length < maxPatternLength )
					{
						try
						{
							arguments.Regexes.Add( word, new Regex( pattern, RegexOptions.IgnoreCase ) );
						}
						catch ( ArgumentException e )
						{
							Console.WriteLine( e.Message );
						}
					}
				}
			}

			this.arguments = arguments;
		}

		public bool IsMatchedItem( Command command, IsMatchedItemArguments arguments )
		{
			if ( !HasAllTags( command, arguments ) )
			{
				return false;
			}

			if ( !IsMatchedItemWithAllWords( command, arguments ) )
			{
				return false;
			}

			return true;
		}

		#region HasAllTags
		private bool HasAllTags( Command command, IsMatchedItemArguments arguments )
		{
			foreach ( string tag in arguments.Tags )
			{
				if ( !HasTag( command, tag ) )
				{
					return false;
				}
			}

			return true;
		}

		private bool HasTag( Command command, string tag )
		{
			foreach ( string one in command.Tags )
			{
				if ( one.StartsWith( tag, StringComparison.CurrentCultureIgnoreCase ) )
				{
					return true;
				}
			}

			return false;
		}
		#endregion

		private bool IsTagFilter( string word )
		{
			return word.StartsWith( @":" );
		}

		private bool IsMatchedItemWithAllWords( Command command, IsMatchedItemArguments arguments )
		{
			foreach ( string word in arguments.Words )
			{
				if ( !IsMatchedItemWithWord( command, arguments, word ) )
				{
					return false;
				}
			}

			return true;
		}

		private bool IsMatchedItemWithWord( Command command, IsMatchedItemArguments arguments, string word )
		{
			if ( arguments.RomajiMatching )
			{
				return IsMatchedItemWithRomajiMatching( command, arguments, word );
			}

			return IsMatchedItemWithoutRomajiMatching( command, arguments, word );
		}

		private bool IsMatchedItemWithoutRomajiMatching( Command command, IsMatchedItemArguments arguments, string word )
		{
			if ( arguments.InitialSearch )
			{
				return command.Name.StartsWith( word, StringComparison.CurrentCultureIgnoreCase );
			}

			return command.Name.ToLower().Contains( word.ToLower() );
		}

		private bool IsMatchedItemWithRomajiMatching( Command command, IsMatchedItemArguments arguments, string word )
		{
			if ( !arguments.Regexes.ContainsKey( word ) )
			{
				return false;
			}

			Regex regex = arguments.Regexes[word];

			try
			{
				return regex.IsMatch( command.Name );
			}
			catch ( Exception e )
			{
				Console.WriteLine( e.Message );
				return false;
			}
		}

		#endregion

		#region SplitFilter
		public string[] SplitFilter( string filter )
		{
			string trimmedFilter = filter.Trim();

			List<string> result = new List<string>();

			while ( 0 < trimmedFilter.Length )
			{
				string element = CutOffFirstElement( ref trimmedFilter );

				if ( element != string.Empty )
				{
					result.Add( element );
				}
			}

			return result.ToArray();
		}

		private string CutOffFirstElement( ref string filter )
		{
			string result;

			if ( filter[0] == CharConstants.DOUBLE_QUOTATION )
			{
				filter = filter.Remove( 0, 1 );

				result = DoCutOffFirstElement( ref filter, CharConstants.DOUBLE_QUOTATION );
			}
			else
			{
				result = DoCutOffFirstElement( ref filter, CharConstants.SPACE );
			}

			filter = filter.Trim();

			return result;
		}

		private string DoCutOffFirstElement( ref string filter, char endOfElement )
		{
			string result;

			int endOfElementIndex = filter.IndexOf( endOfElement );

			if ( endOfElementIndex == -1 )
			{
				result = filter.Trim();

				filter = string.Empty;
			}
			else
			{
				result = filter.Substring( 0, endOfElementIndex );

				filter = filter.Remove( 0, endOfElementIndex + 1 );
			}

			return result;
		} 
		#endregion
	}
}
