﻿using System;
using System.Drawing;
using System.Windows.Forms;

namespace CommandTagLauncher
{
	public partial class OptionsForm : Form
	{
		public OptionsForm( CommonOptions commonOptions, ViewOptions viewOptions )
		{
			InitializeComponent();

			InitializeFontDialog1();

			LoadOptions( commonOptions, viewOptions );
		}

		private void InitializeFontDialog1()
		{
			fontDialog1.ShowEffects = false;

			fontDialog1.ShowApply = false;

			fontDialog1.ShowHelp = false;

			fontDialog1.AllowVerticalFonts = false;
		}

		private void LoadOptions( CommonOptions commonOptions, ViewOptions viewOptions )
		{
			labeledKeysInputToggleMainWindowVisible.KeysInput.KeyData = commonOptions.ToggleMainWindowVisibleKey;
			labeledKeysInputCaptureCurrentProgramKey.KeysInput.KeyData = commonOptions.CaptureCurrentProgramKey;

			labeledKeysInputExecute.KeysInput.KeyData = commonOptions.ExecuteKey;
			labeledKeysInputExecuteWithInputArguments.KeysInput.KeyData = commonOptions.ExecuteWithInputArgumentsKey;
			labeledKeysInputExecuteWithMostRecentUsedArguments.KeysInput.KeyData = commonOptions.ExecuteWithMostRecentUsedArgumentsKey;

			checkBoxHideWhenLoaded.Checked = commonOptions.HideWhenLoaded;
			checkBoxConfirmWhenRemovingItems.Checked = commonOptions.ConfirmWhenRemovingItems;
			checkBoxConfirmWhenClearArgumentsHistory.Checked = commonOptions.ConfirmWhenClearArgumentsHistory;

			labeledComboBoxCommandsArgumentsHistoryMaxCount.ComboBox.Text = commonOptions.CommandsArgumentsHistoryMaxCount.ToString();
			labeledComboBoxMaxShowItemsCount.ComboBox.Text = commonOptions.MaxShowItemsCount.ToString();

			fontDialog1.Font = viewOptions.Font;
			labeledTextBoxFontSample.TextBox.Font = viewOptions.Font;
			labeledTextBoxFontSample.TextBox.ForeColor = viewOptions.TextForeColor;
			labeledTextBoxFontSample.TextBox.BackColor = viewOptions.TextBackColor;
		}

		#region Results
		private CommonOptions commonOptions = new CommonOptions();

		public CommonOptions CommonOptions
		{
			get { return commonOptions; }
		}

		private ViewOptions viewOptions = new ViewOptions();

		public ViewOptions ViewOptions
		{
			get { return viewOptions; }
		}
		#endregion

		private void buttonFont_Click( object sender, EventArgs e )
		{
			if ( fontDialog1.ShowDialog() == DialogResult.OK )
			{
				labeledTextBoxFontSample.TextBox.Font = fontDialog1.Font;
			}
		}

		private void buttonTextForeColor_Click( object sender, EventArgs e )
		{
			labeledTextBoxFontSample.TextBox.ForeColor =
				GetColorWithDialog( labeledTextBoxFontSample.TextBox.ForeColor );
		}

		private void buttonTextBackColor_Click( object sender, EventArgs e )
		{
			labeledTextBoxFontSample.TextBox.BackColor =
				GetColorWithDialog( labeledTextBoxFontSample.TextBox.BackColor );
		}

		private Color GetColorWithDialog( Color initialColor )
		{
			ColorDialog dialog = new ColorDialog();

			dialog.Color = initialColor;

			Color result = initialColor;

			try
			{
				if ( dialog.ShowDialog() == DialogResult.OK )
				{
					result = dialog.Color;
				}
			}
			finally
			{
				dialog.Dispose();
			}

			return result;
		}

		private void okCancelPanel1_OKButton_Click( object sender, EventArgs e )
		{
			if ( IsValid() )
			{
				SetResults();

				this.DialogResult = DialogResult.OK;
			}
			else
			{
				MessageBox.Show( "編集結果が不正です。引数履歴保存数などを確認してください。" );
			}
		}

		private bool IsValid()
		{
			try
			{
				Convert.ToInt32( labeledComboBoxCommandsArgumentsHistoryMaxCount.ComboBox.Text );
				Convert.ToInt32( labeledComboBoxMaxShowItemsCount.ComboBox.Text );
			}
			catch ( Exception )
			{
				return false;
			}

			return true;
		}

		#region SetResults
		private void SetResults()
		{
			SetCommonOption();

			SetViewOption();
		}

		private void SetViewOption()
		{
			viewOptions.Font = fontDialog1.Font;
			viewOptions.Font = labeledTextBoxFontSample.TextBox.Font;
			viewOptions.TextForeColor = labeledTextBoxFontSample.TextBox.ForeColor;
			viewOptions.TextBackColor = labeledTextBoxFontSample.TextBox.BackColor;
		}

		private void SetCommonOption()
		{
			commonOptions.ToggleMainWindowVisibleKey = labeledKeysInputToggleMainWindowVisible.KeysInput.KeyData;
			commonOptions.CaptureCurrentProgramKey = labeledKeysInputCaptureCurrentProgramKey.KeysInput.KeyData;

			commonOptions.ExecuteKey = labeledKeysInputExecute.KeysInput.KeyData;
			commonOptions.ExecuteWithInputArgumentsKey = labeledKeysInputExecuteWithInputArguments.KeysInput.KeyData;
			commonOptions.ExecuteWithMostRecentUsedArgumentsKey = labeledKeysInputExecuteWithMostRecentUsedArguments.KeysInput.KeyData;

			commonOptions.HideWhenLoaded = checkBoxHideWhenLoaded.Checked;
			commonOptions.ConfirmWhenRemovingItems = checkBoxConfirmWhenRemovingItems.Checked;
			commonOptions.ConfirmWhenClearArgumentsHistory = checkBoxConfirmWhenClearArgumentsHistory.Checked;

			commonOptions.CommandsArgumentsHistoryMaxCount =
				Convert.ToInt32( labeledComboBoxCommandsArgumentsHistoryMaxCount.ComboBox.Text );

			commonOptions.MaxShowItemsCount =
				Convert.ToInt32( labeledComboBoxMaxShowItemsCount.ComboBox.Text );
		}
		#endregion
	}
}
