﻿using System;
using System.Collections.Generic;

namespace CommandTagLauncher
{
	public class ChildCommandCollection : IList<Command>
	{
		readonly List<Command> _parameter;

		public ChildCommandCollection()
		{
			_parameter = new List<Command>();
		}

		private void ValidateItem( Command item )
		{
			if ( item == null )
			{
				throw new ArgumentNullException( "item" );
			}

			if ( item is GroupCommand )
			{
				throw new ArgumentException( "item is GroupCommand." );
			}
		}

		#region IList<Command> メンバ

		public int IndexOf( Command item )
		{
			return _parameter.IndexOf( item );
		}

		public void Insert( int index, Command item )
		{
			ValidateItem( item );

			_parameter.Insert( index, item );
		}

		public void RemoveAt( int index )
		{
			_parameter.RemoveAt( index );
		}

		public Command this[int index]
		{
			get
			{
				return _parameter[index];
			}
			set
			{
				ValidateItem( value );

				_parameter[index] = value;
			}
		}

		#endregion

		#region ICollection<Command> メンバ

		public void Add( Command item )
		{
			ValidateItem( item );

			_parameter.Add( item );
		}

		public void Clear()
		{
			_parameter.Clear();
		}

		public bool Contains( Command item )
		{
			return _parameter.Contains( item );
		}

		public void CopyTo( Command[] array, int arrayIndex )
		{
			_parameter.CopyTo( array, arrayIndex );
		}

		public int Count
		{
			get
			{
				return _parameter.Count;
			}
		}

		public bool IsReadOnly
		{
			get
			{
				return ( ( IList<Command> )_parameter ).IsReadOnly;
			}
		}

		public bool Remove( Command item )
		{
			return _parameter.Remove( item );
		}

		#endregion

		#region IEnumerable<Command> メンバ

		public IEnumerator<Command> GetEnumerator()
		{
			return _parameter.GetEnumerator();
		}

		#endregion

		#region IEnumerable メンバ

		System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
		{
			return _parameter.GetEnumerator();
		}

		#endregion
	}
}
