﻿using System;
using System.Collections.Generic;

namespace CommandTagLauncher
{
	public class Commands : IList<Command>, ICommandObserver
	{
		private readonly List<Command> innerList = new List<Command>();

		#region Updated
		public event EventHandler<UpdatedEventArgs> Updated;

		protected virtual void OnUpdated( UpdatedEventArgs e )
		{
			if ( Updated != null )
			{
				Updated( this, e );
			}
		} 
		#endregion

		#region IList<Command> メンバ

		public int IndexOf( Command item )
		{
			return innerList.IndexOf( item );
		}

		public void Insert( int index, Command item )
		{
			innerList.Insert( index, item );

			SetObserverAndArgumentsHistoryMaxCount( item );

			OnUpdated( new UpdatedEventArgs() );
		}

		public void RemoveAt( int index )
		{
			innerList[index].DeleteObserver( this );

			innerList.RemoveAt( index );

			OnUpdated( new UpdatedEventArgs() );
		}

		public Command this[int index]
		{
			get
			{
				return innerList[index];
			}
			set
			{
				innerList[index] = value;

				SetObserverAndArgumentsHistoryMaxCount( value );

				OnUpdated( new UpdatedEventArgs() );
			}
		}

		#endregion

		#region ICollection<Command> メンバ

		public void Add( Command item )
		{
			innerList.Add( item );

			SetObserverAndArgumentsHistoryMaxCount( item );

			OnUpdated( new UpdatedEventArgs() );
		}

		public void Clear()
		{
			foreach ( Command child in innerList )
			{
				child.DeleteObserver( this );
			}

			innerList.Clear();

			OnUpdated( new UpdatedEventArgs() );
		}

		public bool Contains( Command item )
		{
			return innerList.Contains( item );
		}

		public void CopyTo( Command[] array, int arrayIndex )
		{
			innerList.CopyTo( array, arrayIndex );
		}

		public int Count
		{
			get
			{
				return innerList.Count;
			}
		}

		public bool IsReadOnly
		{
			get
			{
				return ( ( IList<Command> )innerList ).IsReadOnly;
			}
		}

		public bool Remove( Command item )
		{
			item.DeleteObserver( this );

			bool result = innerList.Remove( item );

			OnUpdated( new UpdatedEventArgs() );

			return result;
		}

		#endregion

		#region IEnumerable<Command> メンバ

		public IEnumerator<Command> GetEnumerator()
		{
			return innerList.GetEnumerator();
		}

		#endregion

		#region IEnumerable メンバ

		System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
		{
			return innerList.GetEnumerator();
		}

		#endregion

		#region ICommandObserver メンバ

		public void Update( Command command )
		{
			OnUpdated( new UpdatedEventArgs() );
		}

		#endregion

		public void AddRange( IEnumerable<Command> collection )
		{
			innerList.AddRange( collection );

			foreach ( Command command in collection )
			{
				SetObserverAndArgumentsHistoryMaxCount( command );
			}

			OnUpdated( new UpdatedEventArgs() );
		}

		private void SetObserverAndArgumentsHistoryMaxCount( Command command )
		{
			command.AddObserver( this );

			command.SetArgumentsHistoryMaxCount( argumentsHistoryMaxCount );
		}

		public List<Command> ToList()
		{
			List<Command> result = new List<Command>();

			result.AddRange( this );

			return result;
		}

		private int argumentsHistoryMaxCount = int.MaxValue;

		public int GetArgumentsHistoryMaxCount()
		{
			return argumentsHistoryMaxCount;
		}

		public void SetArgumentsHistoryMaxCount( int value )
		{
			if ( argumentsHistoryMaxCount == value )
			{
				return;
			}

			argumentsHistoryMaxCount = value;

			foreach ( Command command in this )
			{
				command.SetArgumentsHistoryMaxCount( value );
			}
		}

		#region Find

		public static Command Find( string name, List<Command> source )
		{
			findTargetName = name;

			return source.Find( HasMatchedName );
		}

		private static string findTargetName;

		private static bool HasMatchedName( Command command )
		{
			return findTargetName == command.Name;
		}
		#endregion
	}
}
