﻿using System;
using System.Diagnostics;
using System.IO;
using System.ComponentModel;

namespace CommandTagLauncher
{
	public class FileCommand : Command
	{
		private string fileName = string.Empty;

		public string FileName
		{
			get { return fileName; }
			set { fileName = value; }
		}

		private string verb = string.Empty;

		[DefaultValue( "" )]
		public string Verb
		{
			get { return verb; }
			set { verb = value; }
		}

		private ProcessWindowStyle windowStyle = ProcessWindowStyle.Normal;

		[DefaultValue( ProcessWindowStyle.Normal )]
		public ProcessWindowStyle WindowStyle
		{
			get { return windowStyle; }
			set { windowStyle = value; }
		}

		private string workingDirectory = string.Empty;

		[DefaultValue( "" )]
		public string WorkingDirectory
		{
			get { return workingDirectory; }
			set { workingDirectory = value; }
		}

		private bool multipleExecution = true;

		[DefaultValue( true )]
		public bool MultipleExecution
		{
			get { return multipleExecution; }
			set { multipleExecution = value; }
		}

		private ProcessPriority processPriority = ProcessPriority.NoControl;

		[DefaultValue( ProcessPriority.NoControl )]
		public ProcessPriority ProcessPriority
		{
			get { return processPriority; }
			set { processPriority = value; }
		}

		protected override void DoExecute( string arguments )
		{
			if ( !File.Exists( FileName ) )
			{
				System.Windows.Forms.MessageBox.Show( FileName + " というファイルが見つかりません。" );

				return;
			}

			ProcessStartInfo processStartInfo = CreateProcessStartInfo( arguments );

			if ( MultipleExecution )
			{
				StartProcess( processStartInfo );
			}
			else
			{
				ActivateOrStartProcess( processStartInfo );
			}
		}

		private void ActivateOrStartProcess( ProcessStartInfo processStartInfo )
		{
			Process sameProcess = FindSameProcess( processStartInfo.FileName );

			if ( sameProcess == null )
			{
				StartProcess( processStartInfo );
			}
			else
			{
				ActivateProcess( sameProcess );
			}
		}

		private void StartProcess( ProcessStartInfo processStartInfo )
		{
			try
			{
				Process process = Process.Start( processStartInfo );

				SetProcessPriority( process );
			}
			catch ( Exception e )
			{
				System.Windows.Forms.MessageBox.Show( e.Message );
			}
		}

		private void SetProcessPriority( Process process )
		{
			if ( ProcessPriority != ProcessPriority.NoControl )
			{
				process.PriorityClass = ConvertToProcessPriorityClass( ProcessPriority );
			}
		}

		private ProcessPriorityClass ConvertToProcessPriorityClass( ProcessPriority processPriority )
		{
			switch ( processPriority )
			{
				case ProcessPriority.RealTime:
					return ProcessPriorityClass.RealTime;
				case ProcessPriority.High:
					return ProcessPriorityClass.High;
				case ProcessPriority.AboveNormal:
					return ProcessPriorityClass.AboveNormal;
				case ProcessPriority.Normal:
					return ProcessPriorityClass.Normal;
				case ProcessPriority.BelowNormal:
					return ProcessPriorityClass.BelowNormal;
				case ProcessPriority.Idle:
					return ProcessPriorityClass.Idle;
			}

			throw new ArgumentException();
		}

		#region CreateProcessStartInfo
		private ProcessStartInfo CreateProcessStartInfo( string arguments )
		{
			ProcessStartInfo processStartInfo = new ProcessStartInfo( FileName, arguments );

			processStartInfo.Verb = Verb;
			processStartInfo.WindowStyle = WindowStyle;
			processStartInfo.WorkingDirectory = GetWorkingDirectory();

			return processStartInfo;
		}

		private string GetWorkingDirectory()
		{
			if ( WorkingDirectory == string.Empty )
			{
				return Path.GetDirectoryName( FileName );
			}

			return WorkingDirectory;
		}

		#endregion

		private Process FindSameProcess( string fileName )
		{
			string name = Path.GetFileNameWithoutExtension( fileName );

			Process[] processes = Process.GetProcessesByName( name );

			if ( processes.Length == 0 )
			{
				return null;
			}

			return processes[0];
		}

		private void ActivateProcess( Process process )
		{
			Tomham.Diagnostics.Utility.ActivateWindow( process );
		}
	}
}
