﻿using System.Collections.Generic;
using System.Windows.Forms;

namespace CommandTagLauncher
{
	// GroupCommandのSerialize/DeSerializeの時だけ使用する

	public class GroupCommandEntry : Command
	{
		private List<string> childCommandNames = new List<string>();

		public List<string> ChildCommandNames
		{
			get { return childCommandNames; }
			set { childCommandNames = value; }
		}

		public void SetGroupCommand( GroupCommand command )
		{
			Name = command.Name;
			Description = command.Description;
			Tags = command.Tags;
			LastExecutedTime = command.LastExecutedTime;
			DefaultArguments = command.DefaultArguments;
			ArgumentsHistory = command.ArgumentsHistory;

			foreach ( Command child in command.Items )
			{
				childCommandNames.Add( child.Name );
			}
		}

		public GroupCommand GetGroupCommand( List<Command> childCommandsReferencesResolveSource )
		{
			GroupCommand result = new GroupCommand()
			{
				Name = Name,
				Description = Description,
				Tags = Tags,
				LastExecutedTime = LastExecutedTime,
				DefaultArguments= DefaultArguments,
				ArgumentsHistory= ArgumentsHistory
			};

			ResolveChildCommandsReferences( result, childCommandsReferencesResolveSource );

			return result;
		}

		private void ResolveChildCommandsReferences( GroupCommand command, List<Command> childCommandsReferencesResolveSource )
		{
			foreach ( string childName in childCommandNames )
			{
				Command child = Commands.Find( childName, childCommandsReferencesResolveSource );

				if ( child == null )
				{
					string message = command.Name + "のメンバである " + childName + " というコマンドが見つかりませんでした。";

					MessageBox.Show( message );
				}
				else
				{
					command.Items.Add( child );
				}
			}
		}

		protected override void DoExecute( string arguments )
		{
			// Do nothing.
		}
	}
}
