﻿using System.Collections.Generic;
using System.Text;
using System.IO;
using Tomham.Xml.Serialization;
using Tomham.Collections.Generic;

namespace CommandTagLauncher
{
	public class CommandsSerializer
	{
		public static Commands Load( string fileName )
		{
			if ( !File.Exists( fileName ) )
			{
				throw new FileNotFoundException();
			}

			CommandsFile commandsFile =
				SimpleXmlSerializer.Deserialize<CommandsFile>( fileName, Encoding.UTF8 );

			Commands result = new Commands();

			result.AddRange( commandsFile.FileCommands.ToArray() );

			List<GroupCommand> groups = GetGroupCommands( commandsFile.GroupCommandEntries, result.ToList() );

			result.AddRange( groups.ToArray() );

			return result;
		}

		private static List<GroupCommand> GetGroupCommands( List<GroupCommandEntry> list, List<Command> childCommandsReferencesResolveSource )
		{
			List<GroupCommand> result = new List<GroupCommand>();

			foreach ( GroupCommandEntry entry in list )
			{
				GroupCommand group = entry.GetGroupCommand( childCommandsReferencesResolveSource );
				result.Add( group );
			}

			return result;
		}

		public static void Save( Commands commands, string fileName )
		{
			List<FileCommand> fileCommands =
				IListUtility.ExtractOfTypeToList<Command, FileCommand>( commands.ToList() );

			List<GroupCommand> groups =
				IListUtility.ExtractOfTypeToList<Command, GroupCommand>( commands.ToList() );

			CommandsFile file = new CommandsFile();

			file.FileCommands = fileCommands;

			file.GroupCommandEntries = GetGroupCommandEntries( groups );

			DoSave( file, fileName );
		}

		private static void DoSave( CommandsFile commandsFile, string fileName )
		{
			Microsoft.Xml.Serialization.GeneratedAssembly.CommandsFileSerializer serializer =
				new Microsoft.Xml.Serialization.GeneratedAssembly.CommandsFileSerializer();

			using ( TextWriter writer = new StreamWriter( fileName, false, Encoding.UTF8 ) )
			{
				serializer.Serialize( writer, commandsFile );
			}
		}

		private static List<GroupCommandEntry> GetGroupCommandEntries( List<GroupCommand> groups )
		{
			List<GroupCommandEntry> result = new List<GroupCommandEntry>();

			foreach ( GroupCommand group in groups )
			{
				GroupCommandEntry entry = new GroupCommandEntry();

				entry.SetGroupCommand( group );

				result.Add( entry );
			}

			return result;
		}
	}
}
