using System;
using System.ComponentModel;
using System.Collections;
using System.Diagnostics;
using System.Windows.Forms;
using CodeProject.Win32;

namespace Tomham.Windows.Forms
{
	/*  This Component based on "System Hotkey Component"
	 *  http://www.codeproject.com/KB/miscctrl/systemhotkey.aspx
	 *		
	 * modified:
	 * Shortcut type changed. System.Windows.Forms.Shorcut to System.Windows.Forms.Keys.
	 * Removed Error event.
	 * Added HotKeyChanged and RegisterHotKeyFailed and UnRegisterHotKeyFailed events.
	 * Added Enabled property.
	 * and a little refactoring.
	 *  
	 *  by tom_ham
	*/

	// Get attention: 
	// WindowsKey( Keys.LWin and Keys.Rwin ) does not work. 
	// I can't make it to work fine.

	public class SystemHotKey : System.ComponentModel.Component, IDisposable
	{
		private System.ComponentModel.Container components = null;
		protected DummyWindowWithEvent m_Window = new DummyWindowWithEvent();	//window for WM_Hotkey Messages

		protected bool isRegistered = false;

		[Browsable(false)]
		public bool IsRegistered
		{
			get { return isRegistered; }
		}

		private bool enabled = true;

		[Category( "Action" )]
		[BrowsableAttribute( true )]
		public bool Enabled
		{
			get { return enabled; }
			set { enabled = value; }
		}

		#region Events

		[Category( "Default" )]
		[BrowsableAttribute( true )]
		public event System.EventHandler HotKeyPressed;

		protected virtual void OnHotKeyPressed( EventArgs e )
		{
			if ( HotKeyPressed != null )
			{
				HotKeyPressed( this, e );
			}
		}

		[Category( "Default" )]
		[BrowsableAttribute( true )]
		public event EventHandler HotKeyChanged;

		protected virtual void OnHotKeyChanged( EventArgs e )
		{
			if ( HotKeyChanged != null )
			{
				HotKeyChanged( this, e );
			}
		}

		[Category( "Default" )]
		[BrowsableAttribute( true )]
		public event EventHandler<RegisterHotKeyFailedEventArgs> RegisterHotKeyFailed;

		protected virtual void OnRegisterHotKeyFailed( RegisterHotKeyFailedEventArgs e )
		{
			if ( RegisterHotKeyFailed != null )
			{
				RegisterHotKeyFailed( this, e );
			}
		}

		[Category( "Default" )]
		[BrowsableAttribute( true )]
		public event EventHandler<UnRegisterHotKeyFailedEventArgs> UnRegisterHotKeyFailed;

		protected virtual void OnUnRegisterHotKeyFailed( UnRegisterHotKeyFailedEventArgs e )
		{
			if ( UnRegisterHotKeyFailed != null )
			{
				UnRegisterHotKeyFailed( this, e );
			}
		}

		#endregion

		public SystemHotKey( System.ComponentModel.IContainer container )
		{
			container.Add( this );
			InitializeComponent();
			m_Window.ProcessMessage += new MessageEventHandler( MessageEvent );
		}

		public SystemHotKey()
		{
			InitializeComponent();
			if ( !DesignMode )
			{
				m_Window.ProcessMessage += new MessageEventHandler( MessageEvent );
			}
		}

		public new void Dispose()
		{
			if ( isRegistered )
			{
				if ( UnRegisterHotKey() )
					System.Diagnostics.Debug.WriteLine( "Unreg: OK" );
			}
			System.Diagnostics.Debug.WriteLine( "Disposed" );
		}

		#region Component Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			components = new System.ComponentModel.Container();
		}
		#endregion

		protected void MessageEvent( object sender, ref Message m, ref bool Handled )
		{	//Handle WM_Hotkey event
			if ( ( m.Msg == ( int )CodeProject.Win32.Msgs.WM_HOTKEY ) && ( m.WParam == ( IntPtr )this.GetType().GetHashCode() ) )
			{
				Handled = true;

				if ( enabled )
				{
					System.Diagnostics.Debug.WriteLine( "HOTKEY pressed!" );
					OnHotKeyPressed( EventArgs.Empty );
				}
			}
		}

		protected bool UnRegisterHotKey()
		{
			return CodeProject.Win32.User32.UnregisterHotKey( m_Window.Handle, this.GetType().GetHashCode() );
		}

		protected bool RegisterHotKey( Keys keys )
		{
			int modifiersNumber = 0;
			Keys modifiers = Keys.None;

			if ( ContainsAltKey( keys ) )
			{
				modifiersNumber += ( int )CodeProject.Win32.Modifiers.MOD_ALT;
				modifiers |= Keys.Alt;
			}
			if ( ContainsShiftKey( keys ) )
			{
				modifiersNumber += ( int )CodeProject.Win32.Modifiers.MOD_SHIFT;
				modifiers |= Keys.Shift;
			}
			if ( ( ContainsControlKey( keys ) ) )
			{
				modifiersNumber += ( int )CodeProject.Win32.Modifiers.MOD_CONTROL;
				modifiers |= Keys.Control;
			}

			return CodeProject.Win32.User32.RegisterHotKey(
				m_Window.Handle,
				this.GetType().GetHashCode(),
				( int )modifiersNumber,
				( ( int )keys ) - ( ( int )modifiers )
				);
		}

		protected bool ContainsShiftKey( Keys keys )
		{
			return ContainsKey( Keys.Shift, keys );
		}

		protected bool ContainsControlKey( Keys keys )
		{
			return ContainsKey( Keys.Control, keys );
		}

		protected bool ContainsAltKey( Keys keys )
		{
			return ContainsKey( Keys.Alt, keys );
		}

		protected bool ContainsKey( Keys key, Keys container )
		{
			return ( key & container ) == key;
		}

		private Keys hotKey;

		public Keys HotKey
		{
			get { return hotKey; }
			set
			{
				SetHotKey( value );
			}
		}

		private void SetHotKey( Keys value )
		{
			if ( hotKey == value )
			{
				return;
			}

			if ( DesignMode )
			{
				DoSetHotKey( value );
				return;
			}

			if ( NeedToUnRegisterOldHotKey( value ) )
			{
				UnRegisterOldHotKey();
			}

			if ( value == Keys.None )
			{
				DoSetHotKey( value );
				return;
			}

			DoRegisterHotKey( value );

			DoSetHotKey( value );
		}

		private void DoSetHotKey( Keys value )
		{
			hotKey = value;

			OnHotKeyChanged( EventArgs.Empty );
		}

		private void DoRegisterHotKey( Keys value )
		{
			if ( RegisterHotKey( value ) )
			{
				System.Diagnostics.Debug.WriteLine( "Reg: OK" );
				isRegistered = true;
			}
			else
			{
				OnRegisterHotKeyFailed( new RegisterHotKeyFailedEventArgs( value ) );
				System.Diagnostics.Debug.WriteLine( "Reg: ERR" );
			}
		}

		private void UnRegisterOldHotKey()
		{
			if ( UnRegisterHotKey() )
			{
				isRegistered = false;
				System.Diagnostics.Debug.WriteLine( "Unreg: OK" );
			}
			else
			{
				OnUnRegisterHotKeyFailed( new UnRegisterHotKeyFailedEventArgs( this.hotKey ) );

				System.Diagnostics.Debug.WriteLine( "Unreg: ERR" );
			}
		}

		private bool NeedToUnRegisterOldHotKey( Keys newHotKey )
		{
			return isRegistered;
		}
	}
}
