﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Tomham.Collections.Generic
{
	public static class IListUtility
	{
		public static void MoveUpItems<T>( this IList<T> list, params int[] targetIndices )
		{
			int excludeToIndex = -1;

			foreach ( int fromIndex in targetIndices )
			{
				Move<T>( fromIndex, fromIndex - 1, list, ref excludeToIndex );
			}
		}

		private static void Move<T>( int fromIndex, int toIndex, IList<T> list, ref int excludeToIndex )
		{
			if ( HasToNeedToMove( toIndex, list, excludeToIndex ) )
			{
				SwapItems( list, fromIndex, toIndex );
			}
			else
			{
				excludeToIndex = fromIndex;
			}
		}

		public static void SwapItems<T>( this IList<T> list, int index1, int index2 )
		{
			T object1 = list[index1];
			T object2 = list[index2];

			list[index2] = object1;
			list[index1] = object2;
		}

		private static bool HasToNeedToMove<T>( int toIndex, IList<T> list, int excludeToIndex )
		{
			if ( !IsValidIndex( toIndex, list ) )
			{
				return false;
			}

			if ( toIndex == excludeToIndex )
			{
				return false;
			}

			return true;
		}

		private static bool IsValidIndex<T>( int index, IList<T> list )
		{
			if ( index < 0 )
			{
				return false;
			}

			if ( list.Count <= index )
			{
				return false;
			}

			return true;
		}

		public static void MoveDownItems<T>( this IList<T> list, params int[] targetIndices )
		{
			int excludeToIndex = -1;

			for ( int i = targetIndices.Length - 1; 0 <= i; i-- )
			{
				int fromIndex = targetIndices[i];

				Move<T>( fromIndex, fromIndex + 1, list, ref excludeToIndex );
			}
		}

		public static IList<T> Extract<T>( IList<T> list, Predicate<T> isMatched )
		{
			if ( list == null )
			{
				throw new ArgumentNullException( "list" );
			}

			if ( isMatched == null )
			{
				throw new ArgumentNullException( "isMatched" );
			}

			List<T> result = new List<T>();

			foreach ( T item in list )
			{
				if ( isMatched( item ) )
				{
					result.Add( item );
				}
			}

			return result;
		}

		public static List<T> ExtractToList<T>( IList<T> list, Predicate<T> isMatched )
		{
			IList<T> extracted = Extract<T>( list, isMatched );

			return ToList<T>( extracted );
		}

		public static IList<TResult> ExtractOfType<T, TResult>( List<T> list )
			where TResult : T
		{
			if ( list == null )
			{
				throw new ArgumentNullException( "list" );
			}

			List<TResult> result = new List<TResult>();

			foreach ( T item in list )
			{
				if ( item is TResult )
				{
					result.Add( ( TResult )item );
				}
			}

			return result;
		}

		public static List<TResult> ExtractOfTypeToList<T, TResult>( List<T> list )
			where TResult : T
		{
			IList<TResult> extracted = IListUtility.ExtractOfType<T, TResult>( list );

			return ToList<TResult>( extracted );
		}

		public static List<T> ToList<T>( IList<T> list )
		{
			if ( list == null )
			{
				throw new ArgumentNullException( "list" );
			}

			List<T> result = new List<T>();

			foreach ( T item in list )
			{
				result.Add( item );
			}

			return result;
		}
	}
}
