using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using System.Runtime.InteropServices;

namespace Tomham.Diagnostics
{
	public static class Utility
	{
		/// <summary>
		/// FileNameArgument؂蕪ProcessStartInfoɊi[ĕԂ
		/// </summary>
		/// <param name="commandLine"></param>
		/// <returns></returns>
		public static ProcessStartInfo CreateProcessStartInfo( string commandLine )
		{
			if ( commandLine == null )
			{
				throw new ArgumentNullException( "commandLine" );
			}

			if ( commandLine == string.Empty )
			{
				throw new ArgumentException( "commandLine == string.Empty" );
			}

			commandLine = commandLine.Trim();

			ProcessStartInfo result = new ProcessStartInfo();

			char doubleQuotation = Convert.ToChar( "\"" );

			if ( commandLine[0] == doubleQuotation )
			{
				if ( commandLine.Length <= 2 )
				{
					throw new ArgumentException( "commandLine is Invalid: " + commandLine );
				}

				int n = commandLine.IndexOf( doubleQuotation, 1 );

				result.FileName = commandLine.Substring( 1, n - 1 );

				try
				{
					result.Arguments = commandLine.Substring( n + 2 );
				}
				catch
				{
					result.Arguments = string.Empty;
				}
			}
			else
			{
				char[] separator = { Convert.ToChar( " " ) };
				string[] splitedCommandLine = commandLine.Split( separator );

				result.FileName = splitedCommandLine[0];

				try
				{
					int firstSpaceIndex = commandLine.IndexOf( " " );

					if ( firstSpaceIndex != -1 )
					{
						result.Arguments = commandLine.Substring( firstSpaceIndex + 1 );
					}
				}
				catch
				{
					result.Arguments = string.Empty;
				}
			}

			result.FileName = result.FileName.Trim();
			result.Arguments = result.Arguments.Trim();

			return result;
		}

		private const int SW_NORMAL = 1;
		private const int SW_SHOW = 5;
		private const int SW_RESTORE = 9;

		[DllImport( "USER32.DLL", CharSet = CharSet.Auto )]
		private static extern int ShowWindow(
			System.IntPtr hWnd,
			int nCmdShow
		);

		[DllImport( "USER32.DLL", CharSet = CharSet.Auto )]
		private static extern int ShowWindowAsync(
			System.IntPtr hWnd,
			int nCmdShow
		);

		[DllImport( "USER32.DLL", CharSet = CharSet.Auto )]
		private static extern bool SetForegroundWindow( System.IntPtr hWnd );

		[DllImport( "user32.dll" )]
		extern static IntPtr GetForegroundWindow();

		[DllImport( "user32.dll" )]
		private static extern uint GetWindowThreadProcessId( IntPtr hWnd, out int lpdwProcessId );

		[DllImport( "user32.dll" )]
		private static extern bool IsIconic( IntPtr hWnd );

		public static bool IsIconic( Process process )
		{
			return IsIconic( process.MainWindowHandle );
		}

		public static Process GetForegroundWindowProcess()
		{
			IntPtr hWnd = GetForegroundWindow();
			int processid;
			GetWindowThreadProcessId( hWnd, out processid );
			return Process.GetProcessById( processid );
		}

		public static void ActivateWindow( Process process )
		{
			if ( IsIconic( process ) )
			{
				ShowWindowAsync( process.MainWindowHandle, SW_RESTORE );
			}
			else
			{
				ShowWindowAsync( process.MainWindowHandle, SW_SHOW );
			}

			SetForegroundWindow( process.MainWindowHandle );
		}

		public static bool ShowPrevProcess()
		{
			Process hThisProcess = Process.GetCurrentProcess();
			Process[] hProcesses = Process.GetProcessesByName( hThisProcess.ProcessName );
			int iThisProcessId = hThisProcess.Id;

			foreach ( Process hProcess in hProcesses )
			{
				if ( hProcess.Id != iThisProcessId )
				{
					ShowWindow( hProcess.MainWindowHandle, SW_NORMAL );
					SetForegroundWindow( hProcess.MainWindowHandle );
					return true;
				}
			}

			return false;
		}
	}
}
