using System.ComponentModel;
using System.Windows.Forms;

namespace Tomham.Windows.Forms
{
    // Original: ShortcutInput component from...
    // Simple steps to enable Hotkey and ShortcutInput user control
    // http://www.codeproject.com/KB/miscctrl/ashsimplehotkeys.aspx

    // Modified Points:
    // CharCode base to Keys base.
    // Renamed component and several members.

    public class KeysInput : UserControl
    {
        #region Public Properties

        public byte MinModifiers;

        public Keys KeyCode
        {
            get
            {
                if ( 0 <= comboBoxKeyCode.SelectedIndex )
                {
                    return (Keys) comboBoxKeyCode.SelectedItem;
                }
                else
                {
                    return Keys.None;
                }
            }
            set
            {
                if ( comboBoxKeyCode.Items.Contains( value ) )
                {
                    comboBoxKeyCode.SelectedItem = value;
                }
                else
                {
                    comboBoxKeyCode.SelectedIndex = -1;
                }
            }
        }

        public Keys KeyData
        {
            get
            {
                Keys result = KeyCode;

                if ( comboBoxShift.Checked )
                {
                    result |= Keys.Shift;
                }

                if ( comboBoxControl.Checked )
                {
                    result |= Keys.Control;
                }

                if ( comboBoxAlt.Checked )
                {
                    result |= Keys.Alt;
                }

                return result;
            }
            set
            {
                Shift = KeysHelper.ContainsShift( value );
                Control = KeysHelper.ContainsControl( value );
                Alt = KeysHelper.ContainsAlt( value );

                KeyCode = value & Keys.KeyCode;
            }
        }

        public bool Shift
        {
            get { return comboBoxShift.Checked; }
            set { comboBoxShift.Checked = value; }
        }

        public bool Control
        {
            get { return comboBoxControl.Checked; }
            set { comboBoxControl.Checked = value; }
        }

        public bool Alt
        {
            get { return comboBoxAlt.Checked; }
            set { comboBoxAlt.Checked = value; }
        }

        public bool IsValid
        {
            get
            {
                byte ModCount = 0;
                ModCount += (byte) ( ( Shift ) ? 1 : 0 );
                ModCount += (byte) ( ( Control ) ? 1 : 0 );
                ModCount += (byte) ( ( Alt ) ? 1 : 0 );
                if ( ModCount < MinModifiers )
                {
                    return false;
                }
                else
                {
                    return true;
                }
            }
        }

        #endregion

        /// <summary> 
        /// Required designer variable.
        /// </summary>
        private readonly Container components;

        private CheckBox comboBoxAlt;
        private CheckBox comboBoxControl;
        private ComboBox comboBoxKeyCode;
        private CheckBox comboBoxShift;

        public KeysInput()
        {
            // This call is required by the Windows.Forms Form Designer.
            InitializeComponent();

            InitializeComboBoxKeyCode();
        }

        private void InitializeComboBoxKeyCode()
        {
            AddAlphabetsToComboBoxKeyCode();

            AddNumbersToComboBoxKeyCode();

            AddFunctionKeysToComboBoxKeyCode();

            AddEtcToComboBoxKeyCode();
        }

        private void AddAlphabetsToComboBoxKeyCode()
        {
            AddItemsToComboBoxKeyCode( 65, 90 );
        }

        private void AddNumbersToComboBoxKeyCode()
        {
            AddItemsToComboBoxKeyCode( 48, 57 );
        }

        private void AddEtcToComboBoxKeyCode()
        {
            AddItemToComboBoxKeyCode( Keys.Back );
            AddItemToComboBoxKeyCode( Keys.Enter );
            AddItemToComboBoxKeyCode( Keys.Pause );
            AddItemToComboBoxKeyCode( Keys.Escape );

            AddItemToComboBoxKeyCode( Keys.IMEConvert );
            AddItemToComboBoxKeyCode( Keys.IMENonconvert );

            AddItemsToComboBoxKeyCode( 0x20, 0x28 );
            AddItemsToComboBoxKeyCode( 0x2C, 0x2E );

            AddItemsToComboBoxKeyCode( 0xba, 0xBF );

            AddItemToComboBoxKeyCode( 0xC0 );

            AddItemsToComboBoxKeyCode( 0xDB, 0xDE );
            AddItemToComboBoxKeyCode( Keys.Oem102 );

            AddItemsToComboBoxKeyCode( 0x90, 0x91 );

            AddNumPadKeysToComboBoxKeyCode();
        }

        private void AddNumPadKeysToComboBoxKeyCode()
        {
            AddItemsToComboBoxKeyCode( 0x60, 0x6F );
        }

        private void AddFunctionKeysToComboBoxKeyCode()
        {
            AddItemsToComboBoxKeyCode( 0x70, 0x87 );
        }

        private void AddItemToComboBoxKeyCode( Keys keys )
        {
            comboBoxKeyCode.Items.Add( keys );
        }

        private void AddItemToComboBoxKeyCode( int code )
        {
            AddItemToComboBoxKeyCode( (Keys) code );
        }

        private void AddItemsToComboBoxKeyCode( int startCode, int endCode )
        {
            for ( int i = startCode; i <= endCode; i++ )
            {
                AddItemToComboBoxKeyCode( i );
            }
        }

        /// <summary> 
        /// Clean up any resources being used.
        /// </summary>
        protected override void Dispose( bool disposing )
        {
            if ( disposing )
            {
                if ( components != null )
                {
                    components.Dispose();
                }
            }
            base.Dispose( disposing );
        }

        #region Component Designer generated code

        /// <summary> 
        /// Required method for Designer support - do not modify 
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent()
        {
            this.comboBoxShift = new System.Windows.Forms.CheckBox();
            this.comboBoxControl = new System.Windows.Forms.CheckBox();
            this.comboBoxAlt = new System.Windows.Forms.CheckBox();
            this.comboBoxKeyCode = new System.Windows.Forms.ComboBox();
            this.SuspendLayout();
            // 
            // comboBoxShift
            // 
            this.comboBoxShift.Location = new System.Drawing.Point( 8, 8 );
            this.comboBoxShift.Name = "comboBoxShift";
            this.comboBoxShift.Size = new System.Drawing.Size( 56, 24 );
            this.comboBoxShift.TabIndex = 0;
            this.comboBoxShift.Text = "Shift";
            // 
            // comboBoxControl
            // 
            this.comboBoxControl.Location = new System.Drawing.Point( 64, 8 );
            this.comboBoxControl.Name = "comboBoxControl";
            this.comboBoxControl.Size = new System.Drawing.Size( 64, 24 );
            this.comboBoxControl.TabIndex = 1;
            this.comboBoxControl.Text = "Control";
            // 
            // comboBoxAlt
            // 
            this.comboBoxAlt.Location = new System.Drawing.Point( 136, 8 );
            this.comboBoxAlt.Name = "comboBoxAlt";
            this.comboBoxAlt.Size = new System.Drawing.Size( 40, 24 );
            this.comboBoxAlt.TabIndex = 2;
            this.comboBoxAlt.Text = "Alt";
            // 
            // comboBoxKeyCode
            // 
            this.comboBoxKeyCode.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
            this.comboBoxKeyCode.Location = new System.Drawing.Point( 182, 8 );
            this.comboBoxKeyCode.Name = "comboBoxKeyCode";
            this.comboBoxKeyCode.Size = new System.Drawing.Size( 127, 20 );
            this.comboBoxKeyCode.TabIndex = 4;
            // 
            // KeysInput
            // 
            this.Controls.Add( this.comboBoxKeyCode );
            this.Controls.Add( this.comboBoxAlt );
            this.Controls.Add( this.comboBoxControl );
            this.Controls.Add( this.comboBoxShift );
            this.Name = "KeysInput";
            this.Size = new System.Drawing.Size( 318, 38 );
            this.ResumeLayout( false );
        }

        #endregion
    }
}