﻿using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;

namespace Tomham.Windows.Forms
{
	public partial class LabeledControl : UserControl
	{
		private Control ContentControl;
		private TitleSectionLocation titleSectionLocation = TitleSectionLocation.Top;

		public LabeledControl()
		{
			InitializeComponent();

			InitializeTableLaoutPanelRoot();
		}

		[Browsable( false )]
		public ColumnStyle FirstColumnStyle
		{
			get { return tableLayoutPanelRoot.ColumnStyles[0]; }
		}

		[Browsable( false )]
		public RowStyle FirstRowStyle
		{
			get { return tableLayoutPanelRoot.RowStyles[0]; }
		}

		[Category( "Appearance" )]
		[Browsable( true )]
		public string TitleText
		{
			get { return labelTitle.Text; }
			set { labelTitle.Text = value; }
		}

		[Category( "Appearance" )]
		[Browsable( true )]
		public Font TitleFont
		{
			get { return labelTitle.Font; }
			set { labelTitle.Font = value; }
		}

		[Category( "Appearance" )]
		[Browsable( true )]
		public Color TitleForeColor
		{
			get { return labelTitle.ForeColor; }
			set { labelTitle.ForeColor = value; }
		}

		[Category( "Appearance" )]
		[Browsable( true )]
		public Color TitleBackColor
		{
			get { return labelTitle.BackColor; }
			set { labelTitle.BackColor = value; }
		}

		[Category( "Layout" )]
		[Browsable( true )]
		public DockStyle TitleDock
		{
			get { return labelTitle.Dock; }
			set { labelTitle.Dock = value; }
		}

		[Category( "Layout" )]
		[Browsable( true )]
		public AnchorStyles TitleAnchor
		{
			get { return labelTitle.Anchor; }
			set { labelTitle.Anchor = value; }
		}

		[Category( "Layout" )]
		[Browsable( true )]
		[DefaultValue( TitleSectionLocation.Top )]
		public TitleSectionLocation TitleSectionLocation
		{
			get { return titleSectionLocation; }
			set { SetTitleSectionLocation( value ); }
		}

		private void InitializeTableLaoutPanelRoot()
		{
			FirstColumnStyle.SizeType = SizeType.AutoSize;
			FirstRowStyle.SizeType = SizeType.AutoSize;
		}

		private void SetTitleSectionLocation( TitleSectionLocation value )
		{
			titleSectionLocation = value;

			if ( ContentControl == null )
			{
				return;
			}

			ClearContentSection();

			DoSetContent();
		}

		private void ClearContentSection()
		{
			ClearContentSectionsColumnAndRow();

			tableLayoutPanelRoot.Controls.Remove( ContentControl );
		}

		private void ClearContentSectionsColumnAndRow()
		{
			tableLayoutPanelRoot.ColumnCount = 1;
			tableLayoutPanelRoot.RowCount = 1;

			if ( 2 <= tableLayoutPanelRoot.ColumnStyles.Count )
			{
				tableLayoutPanelRoot.ColumnStyles.RemoveAt( 1 );
			}
			if ( 2 <= tableLayoutPanelRoot.RowStyles.Count )
			{
				tableLayoutPanelRoot.RowStyles.RemoveAt( 1 );
			}
		}

		private void DoSetContent()
		{
			if ( titleSectionLocation == TitleSectionLocation.Left )
			{
				tableLayoutPanelRoot.ColumnCount = 2;
				tableLayoutPanelRoot.ColumnStyles.Add( new ColumnStyle( SizeType.Percent, 100f ) );
				tableLayoutPanelRoot.Controls.Add( ContentControl, 1, 0 );
				ContentControl.Anchor = AnchorStyles.Left | AnchorStyles.Right;
			}
			else
			{
				tableLayoutPanelRoot.RowCount = 2;
				tableLayoutPanelRoot.RowStyles.Add( new RowStyle( SizeType.Percent, 100f ) );
				tableLayoutPanelRoot.Controls.Add( ContentControl, 0, 1 );
				ContentControl.Anchor = AnchorStyles.Left | AnchorStyles.Top;
			}

			ContentControl.Dock = DockStyle.Fill;
		}

		public void SetContent( Control content )
		{
			if ( ContentControl != null )
			{
				throw new InvalidOperationException( "SetContentは2度呼び出さないでください。" );
			}

			ContentControl = content;

			DoSetContent();

			content.TabIndex = 1;
			AdjustContentSize();
		}

		private void tableLayoutPanelRoot_SizeChanged( object sender, EventArgs e )
		{
			if ( ContentControl != null )
			{
				AdjustContentSize();
			}
		}

		private void AdjustContentSize()
		{
			ContentControl.Size = new Size( 0, 0 );

			ContentControl.Dock = DockStyle.Fill;
		}
	}
}