﻿using System.Windows.Forms;

namespace Tomham.Windows.Forms
{
	public static class ListBoxUtilities
	{
		public static void MoveUpSelectedItems( this ListBox listBox )
		{
			int[] selectedIndices = GetSelectedIndicesCopy( listBox );

			listBox.BeginUpdate();

			foreach ( int index in selectedIndices )
			{
				MoveUp( index, listBox );
			}

			listBox.EndUpdate();
		}

		private static void MoveUp( int fromIndex, ListBox listBox )
		{
			int toIndex = fromIndex - 1;

			if ( HasToNeedToMove( toIndex, listBox ) )
			{
				SwapItems( listBox, fromIndex, toIndex );
			}
		}

		public static void SwapItems( this ListBox listBox, int index1, int index2 )
		{
			listBox.BeginUpdate();

			bool oldIndex1Selected = listBox.GetSelected( index1 );
			bool oldIndex2Selected = listBox.GetSelected( index2 );

			DoSwapItems( listBox, index1, index2 );

			listBox.SetSelected( index1, oldIndex2Selected );
			listBox.SetSelected( index2, oldIndex1Selected );

			listBox.EndUpdate();
		}

		private static void DoSwapItems( ListBox listBox, int index1, int index2 )
		{
			object object1 = listBox.Items[index1];
			object object2 = listBox.Items[index2];

			listBox.Items[index2] = object1;
			listBox.Items[index1] = object2;
		}

		private static bool HasToNeedToMove( int toIndex, ListBox listBox )
		{
			if ( !IsValidIndex( toIndex, listBox ) )
			{
				return false;
			}

			if ( listBox.GetSelected( toIndex ) )
			{
				return false;
			}

			return true;
		}

		private static bool IsValidIndex( int index, ListBox listBox )
		{
			if ( index < 0 )
			{
				return false;
			}

			if ( listBox.Items.Count <= index )
			{
				return false;
			}

			return true;
		}

		private static int[] GetSelectedIndicesCopy( ListBox listBox )
		{
			int[] selectedIndices = new int[listBox.SelectedItems.Count];

			listBox.SelectedIndices.CopyTo( selectedIndices, 0 );

			return selectedIndices;
		}

		public static void MoveDownSelectedItems( this ListBox listBox )
		{
			int[] selectedIndices = GetSelectedIndicesCopy( listBox );

			listBox.BeginUpdate();

			for ( int i = selectedIndices.Length - 1; 0 <= i; i-- )
			{
				int index = selectedIndices[i];

				MoveDown( index, listBox );
			}

			listBox.EndUpdate();
		}

		private static void MoveDown( int fromIndex, ListBox listBox )
		{
			int toIndex = fromIndex + 1;

			if ( HasToNeedToMove( toIndex, listBox ) )
			{
				SwapItems( listBox, fromIndex, toIndex );
			}
		}

		public static void RemoveSelectedItems( this ListBox listBox )
		{
			while ( 0 <= listBox.SelectedIndex )
			{
				listBox.Items.RemoveAt( listBox.SelectedIndex );
			}
		}
	}
}